library(tidyverse)
library(caret)
library(caTools)
library(RRF)
library(kernlab)

#setting the working directory----
setwd("C:/Users/annel/OneDrive - Kruvelab/Kruvelab/Teaching/Data Analysis/Labs/4. Ensemble learnign")

#reading in data ----
toxicity_dataset <- read_delim("bkcs11835-sup-0002-tables1.csv",
                               delim = ",",
                               col_names = TRUE)
descs <- read_delim("descs_pNOEC.csv",
                    delim = ",",
                    col_names = TRUE) %>%
  na.omit()

#cleaning descriptor data----
descs_without_SMILES <- descs %>%
  select(-SMILES, -bruto_formula)

descs_without_SMILES <- descs_without_SMILES %>%
  select(-nearZeroVar(descs_without_SMILES))

descs_without_SMILES <- descs_without_SMILES %>% 
  select(-findCorrelation(cor(descs_without_SMILES), cutoff = 0.8))

descs_with_SMILES <- descs_without_SMILES %>%
  mutate(SMILES = descs$SMILES)

toxicity_dataset <- toxicity_dataset %>%
  left_join(descs_with_SMILES) %>%
  na.omit()

#splitting data----
set.seed(123)
split_train_test_validation <- sample.split(toxicity_dataset$pNOAEC, 
                                            SplitRatio = 0.8)

train_test <- toxicity_dataset %>%
  filter(split_train_test_validation == TRUE)

validation <- toxicity_dataset %>%
  filter(split_train_test_validation != TRUE)

set.seed(987)
split_train_test <- sample.split(train_test$pNOAEC, 
                                 SplitRatio = 0.75)
train <- train_test %>%
  filter(split_train_test == TRUE)

test <- train_test %>%
  filter(split_train_test != TRUE)

#training Random Forest

fitControl <- trainControl(method = "repeatedcv",
                           number = 5,
                           repeats = 2)
regressor_RF <- train(pNOAEC ~ .,
                      data = train %>%
                        select(-SMILES, -CAS),
                      method = "RRF",
                      trControl = fitControl)

train_test_with_predicted_pNOAEC <- train_test %>%
  mutate(pNOAEC_pred_RF = predict(regressor_RF, newdata = train_test),
         split = split_train_test)

train_test_with_predicted_pNOAEC <- train_test_with_predicted_pNOAEC %>%
  select(CAS, SMILES, pNOAEC, pNOAEC_pred_RF, everything())

plot_rf <- ggplot(data = train_test_with_predicted_pNOAEC) +
  geom_abline(slope = 1, intercept = 0) + 
  geom_point(mapping = aes(x = pNOAEC,
                           y = pNOAEC_pred_RF,
                           color = split,
                           text = CAS)) +
  facet_wrap(~split) +
  labs(x = "Measured p NOEAC", y = "Predicted pNOAEC") +
  xlim(1, 10) +
  ylim(1, 10) +
  my_theme

ggsave("my_scatterplot_20201112.pdf",
       width = 16.5,
       height = 8,
       units = "cm")

interactive_plot <- ggplotly(plot_rf)
interactive_plot

htmlwidgets::saveWidget(interactive_plot,
                        file = "scatterplot_interactive.html")

rmse_RF <- RMSE((train_test_with_predicted_pNOAEC %>% filter(split == FALSE))$pNOAEC,
                (train_test_with_predicted_pNOAEC %>% filter(split == FALSE))$pNOAEC_pred_RF)

importance_RF <- RRF::importance(regressor_RF$finalModel)

#SVM models----

regressor_SVM_lin <- train(pNOAEC ~ .,
                           data = train %>%
                             select(-SMILES, -CAS),
                           method = "svmLinear2",     #svmLinear2, svmPoly or #svmRarial
                           trControl = fitControl)

regressor_SVM_poly <- train(pNOAEC ~ .,
                            data = train %>%
                              select(-SMILES, -CAS),
                            method = "svmPoly",     #svmLinear2, svmPoly or #svmRarial
                            trControl = fitControl)
regressor_SVM_rad <- train(pNOAEC ~ .,
                           data = train %>%
                             select(-SMILES, -CAS),
                           method = "svmRadial",     #svmLinear2, svmPoly or #svmRarial
                           trControl = fitControl)

train_test_with_predicted_pNOAEC <- train_test_with_predicted_pNOAEC %>%
  mutate(pNOAEC_pred_SVM_lin = predict(regressor_SVM_lin, newdata = train_test_with_predicted_pNOAEC),
         pNOAEC_pred_SVM_poly = predict(regressor_SVM_poly, newdata = train_test_with_predicted_pNOAEC),
         pNOAEC_pred_SVM_rad = predict(regressor_SVM_rad, newdata = train_test_with_predicted_pNOAEC))

plot_svm_lin <- ggplot(data = train_test_with_predicted_pNOAEC) +
  geom_abline(slope = 1, intercept = 0) + 
  geom_point(mapping = aes(x = pNOAEC,
                           y = pNOAEC_pred_SVM_lin,
                           color = split,
                           text = CAS)) +
  facet_wrap(~split) +
  labs(x = "Measured p NOEAC", y = "Predicted pNOAEC") +
  xlim(1, 10) +
  ylim(1, 10) +
  my_theme
plot_svm_lin


plot_svm_poly <- ggplot(data = train_test_with_predicted_pNOAEC) +
  geom_abline(slope = 1, intercept = 0) + 
  geom_point(mapping = aes(x = pNOAEC,
                           y = pNOAEC_pred_SVM_poly,
                           color = split,
                           text = CAS)) +
  facet_wrap(~split) +
  labs(x = "Measured p NOEAC", y = "Predicted pNOAEC") +
  xlim(1, 10) +
  ylim(1, 10) +
  my_theme
plot_svm_poly


plot_svm_rad <- ggplot(data = train_test_with_predicted_pNOAEC) +
  geom_abline(slope = 1, intercept = 0) + 
  geom_point(mapping = aes(x = pNOAEC,
                           y = pNOAEC_pred_SVM_rad,
                           color = split,
                           text = CAS)) +
  facet_wrap(~split) +
  labs(x = "Measured p NOEAC", y = "Predicted pNOAEC") +
  xlim(1, 10) +
  ylim(1, 10) +
  my_theme
plot_svm_rad

rmse_SVM_lin <- RMSE((train_test_with_predicted_pNOAEC %>% filter(split == FALSE))$pNOAEC,
                (train_test_with_predicted_pNOAEC %>% filter(split == FALSE))$pNOAEC_pred_SVM_lin)

rmse_SVM_poly <- RMSE((train_test_with_predicted_pNOAEC %>% filter(split == FALSE))$pNOAEC,
                     (train_test_with_predicted_pNOAEC %>% filter(split == FALSE))$pNOAEC_pred_SVM_poly)

rmse_SVM_rad <- RMSE((train_test_with_predicted_pNOAEC %>% filter(split == FALSE))$pNOAEC,
                     (train_test_with_predicted_pNOAEC %>% filter(split == FALSE))$pNOAEC_pred_SVM_rad)

#predict validation
validation_with_prediction <- validation %>%
  mutate(pNOAEC_pred_RF = predict(regressor_RF, newdata = validation))


ggplot(data = validation_with_prediction) +
  geom_abline(slope = 1, intercept = 0) + 
  geom_abline(slope = 1, intercept = 1) +
  geom_abline(slope = 1, intercept = -1) +
  geom_point(mapping = aes(x = pNOAEC,
                           y = pNOAEC_pred_RF),
             color = "blue") +
  labs(x = "Measured p NOEAC", y = "Predicted pNOAEC") +
  xlim(1, 10) +
  ylim(1, 10) +
  my_theme

rmse_validation_RF <- RMSE(validation_with_prediction$pNOAEC,
                     validation_with_prediction$pNOAEC_pred_RF)
